#!/bin/bash

####### VARIABLES ########

APP_NAME=dsec-agent
INIT="init"
COMM=$(cat /proc/1/comm)
DIR=$(dirname "${0}")
AGENT_HOME_RELATIVE_PATH=$DIR/../../
cd $AGENT_HOME_RELATIVE_PATH
AGENT_HOME_FULL_PATH=$(pwd)

####### COMMON FUNCTIONS ########

echo detected service manager: $COMM

ask_to_overwrite(){
	read -r -p "$APP_NAME is already installed as a service. Do you want to overwrite? [y/N] " response
	case "$response" in
		[yY][eE][sS]|[yY]) 
			return 0
			;;
		*)
	esac
	return 1
}

do_post_install_operations(){
    if [ $(cat /etc/passwd | grep $APP_NAME | wc -l) = 0 ]; then
	    sudo useradd -r -s /bin/false $APP_NAME
    fi
    if [ ! -d /var/log/$APP_NAME ]; then
	    sudo mkdir -p /var/log/$APP_NAME
    fi
	sudo chown $APP_NAME:$APP_NAME /var/log/$APP_NAME
	sudo chown $APP_NAME:$APP_NAME $AGENT_HOME_FULL_PATH
}

####### SYSTEMD FUNCTIONS ########

is_app_exists_in_systemd(){
	if [ -f /etc/systemd/system/$APP_NAME.service ]; then
		return 0
	fi
	return 1
}

start_systemd_service(){
	echo Trying to start $APP_NAME ...
	sudo systemctl daemon-reload
	sudo systemctl start $APP_NAME.service
	echo By default, logs will be located at /var/log/$APP_NAME/
	sudo systemctl status $APP_NAME.service
}

stop_systemd_service(){
	echo Trying to stop $APP_NAME ...
	sudo systemctl stop $APP_NAME.service
	sudo systemctl status $APP_NAME.service
}

install_systemd_service(){
	echo Trying to install $APP_NAME ...
	cp  config/unix/$APP_NAME\_systemd_template.service $APP_NAME.service
	sed -i -e "s|AGENT_HOME|$AGENT_HOME_FULL_PATH|g" $APP_NAME.service
	sed -i -e "s|USER_NAME|$APP_NAME|g" $APP_NAME.service
	sudo mv $APP_NAME.service /etc/systemd/system/
	do_post_install_operations
	if is_app_exists_in_systemd; then
	   echo $APP_NAME has been installed successfully at $AGENT_HOME_FULL_PATH
	   echo By default, logs will be located at /var/log/$APP_NAME/
	else
	   echo An error occured while installing $APP_NAME
	fi
}

uninstall_systemd_service(){
	echo Trying to uninstall $APP_NAME ...
	sudo rm /etc/systemd/system/$APP_NAME.service
	if ! is_app_exists_in_systemd; then
	   echo $APP_NAME has been uninstalled successfully
	else
	   echo An error occured while uninstalling $APP_NAME
	fi
}

####### INITD FUNCTIONS ########

is_app_exists_in_initd(){
	if [ -f /etc/init.d/$APP_NAME ]; then
		return 0
	fi
	return 1
}

start_initd_service(){
	echo Trying to start $APP_NAME ...
	sudo service $APP_NAME start
	echo By default, logs will be located at /var/log/$APP_NAME/
}

stop_initd_service(){
	echo Trying to stop $APP_NAME ...
	sudo service $APP_NAME stop
	sudo service $APP_NAME status
}

install_initd_service(){
	echo Trying to install $APP_NAME ...
	cp  config/unix/$APP_NAME\_initd_template $APP_NAME
	sed -i -e "s|AGENT_HOME|$AGENT_HOME_FULL_PATH|g" $APP_NAME
	sed -i -e "s|USER_NAME|$APP_NAME|g" $APP_NAME
	sudo mv $APP_NAME /etc/init.d/
	chmod u+x /etc/init.d/$APP_NAME
	do_post_install_operations
	if is_app_exists_in_initd; then
	   echo $APP_NAME has been installed successfully at $AGENT_HOME_FULL_PATH
	   echo By default, logs will be located at /var/log/$APP_NAME/
	else
	   echo An error occured while installing $APP_NAME
	fi
}

uninstall_initd_service(){
	echo Trying to uninstall $APP_NAME ...
	sudo rm /etc/init.d/$APP_NAME
	if ! is_app_exists_in_initd; then
	   echo $APP_NAME has been uninstalled successfully
	else
	   echo An error occured while uninstalling $APP_NAME
	fi
}
